#!/usr/bin/env python3
#
# Copyright 2020 Ubiq Security, Inc., Proprietary and All Rights Reserved.
#
# NOTICE:  All information contained herein is, and remains the property
# of Ubiq Security, Inc. The intellectual and technical concepts contained
# herein are proprietary to Ubiq Security, Inc. and its suppliers and may be
# covered by U.S. and Foreign Patents, patents in process, and are
# protected by trade secret or copyright law. Dissemination of this
# information or reproduction of this material is strictly forbidden
# unless prior written permission is obtained from Ubiq Security, Inc.
#
# Your use of the software is expressly conditioned upon the terms
# and conditions available at:
#
#     https://ubiqsecurity.com/legal
#

import configparser
import os
from . import UBIQ_HOST

class credentialsInfo:

    def __init__(self, access_key_id, secret_signing_key, secret_crypto_access_key, host):
        self.__access_key_id = access_key_id
        self.__secret_signing_key = secret_signing_key
        self.__secret_crypto_access_key = secret_crypto_access_key
        self.__host = host
    
    def get_access_key_id(self):
        return self.__access_key_id
    access_key_id=property(get_access_key_id)

    def get_secret_signing_key(self):
        return self.__secret_signing_key
    secret_signing_key=property(get_secret_signing_key)

    def get_secret_crypto_access_key(self):
        return self.__secret_crypto_access_key
    secret_crypto_access_key = property(get_secret_crypto_access_key)

    def get_host(self):
        return self.__host
    host = property(get_host)
    
    def set(self):
        return (self.__access_key_id != None and self.__secret_signing_key != None and self.__secret_crypto_access_key != None)

class configCredentials(credentialsInfo):

    def load_config_file(self, config_file, profile):
        config = configparser.ConfigParser()
        config.read(config_file)

        # Create empty dictionaries for the default and supplied profile
        d = {}
        p = {}
        
        # get the default profile if there is one
        if (config.has_section('default')):
            d = config['default']
        
        # get the supplied profile if there is one
        if (config.has_section(profile)):
            p = config[profile]
        
        # Use given profile if it is available, otherwise use default.
        self.__access_key_id= p.get('access_key_id', d.get('access_key_id'))
        self.__secret_signing_key = p.get('secret_signing_key', d.get('secret_signing_key'))
        self.__secret_crypto_access_key = p.get('secret_crypto_access_key', d.get('secret_crypto_access_key'))
        self.__host = p.get('SERVER', d.get('SERVER', UBIQ_HOST))


    def __init__(self, config_file = None, profile = "default"):

        self.__access_key_id = None
        self.__secret_signing_key = None
        self.__secret_crypto_access_key = None
        self.__host = None

        if (config_file == None):
            from os.path import expanduser
            home = expanduser("~")
            config_file = os.path.join(home, ".ubiq", "credentials")
            
        if os.path.exists(config_file):
                self.load_config_file(config_file, profile)

        credentialsInfo.__init__(self, self.__access_key_id , self.__secret_signing_key, self.__secret_crypto_access_key, self.__host)

        if (not self.set()):        
            raise RuntimeError("Unable to open config file '{0}' or contains missing values.  ".format(config_file))


class credentials(credentialsInfo):

    def __init__(self, access_key_id = None, secret_signing_key = None, secret_crypto_access_key = None, host = UBIQ_HOST):
        # If supplied value is None, use ENV variable, otherwise use supplied value.
        # If env value isn't set, use the supplied value anyways (None) but prevent an exception
        self.__access_key_id = (access_key_id, os.getenv('UBIQ_ACCESS_KEY_ID', access_key_id)) [access_key_id == None]
        self.__secret_signing_key = (secret_signing_key, os.getenv('UBIQ_SECRET_SIGNING_KEY', secret_signing_key)) [secret_signing_key == None]
        self.__secret_crypto_access_key = (secret_crypto_access_key, os.getenv('UBIQ_SECRET_CRYPTO_ACCESS_KEY', secret_crypto_access_key)) [secret_crypto_access_key == None]

        credentialsInfo.__init__(self, self.__access_key_id,
                              self.__secret_signing_key,
                              self.__secret_crypto_access_key, host)



#!/usr/bin/env python3
#
# Copyright 2020 Ubiq Security, Inc., Proprietary and All Rights Reserved.
#
# NOTICE:  All information contained herein is, and remains the property
# of Ubiq Security, Inc. The intellectual and technical concepts contained
# herein are proprietary to Ubiq Security, Inc. and its suppliers and may be
# covered by U.S. and Foreign Patents, patents in process, and are
# protected by trade secret or copyright law. Dissemination of this
# information or reproduction of this material is strictly forbidden
# unless prior written permission is obtained from Ubiq Security, Inc.
#
# Your use of the software is expressly conditioned upon the terms
# and conditions available at:
#
#     https://ubiqsecurity.com/legal
#

UBIQ_HOST = 'api.ubiqsecurity.com:8811'

from .auth import http_auth
from .encrypt import encryption, encrypt
from .decrypt import decryption, decrypt
from .credentials import credentials, configCredentials



#
# Copyright 2020 Ubiq Security, Inc., Proprietary and All Rights Reserved.
#
# NOTICE:  All information contained herein is, and remains the property
# of Ubiq Security, Inc. The intellectual and technical concepts contained
# herein are proprietary to Ubiq Security, Inc. and its suppliers and may be
# covered by U.S. and Foreign Patents, patents in process, and are
# protected by trade secret or copyright law. Dissemination of this
# information or reproduction of this material is strictly forbidden
# unless prior written permission is obtained from Ubiq Security, Inc.
#
# Your use of the software is expressly conditioned upon the terms
# and conditions available at:
#
#     https://ubiqsecurity.com/legal
#

import os
import setuptools

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

os.chdir(here)


with open(os.path.join(here, "README.md"), "r", encoding="utf-8") as f:
    long_description = f.read()

version_contents = {}
with open(os.path.join(here, "ubiq_security", "version.py"), "r", encoding="utf-8") as f:
    exec(f.read(), version_contents)

requirementPath = os.path.join(here, "requirements.txt")
install_requires = [] 
if os.path.isfile(requirementPath):
    with open(requirementPath, "r", encoding="utf-8") as f:
        install_requires = f.read().splitlines()

setuptools.setup(
    name="ubiq-security",
    version=version_contents["VERSION"],
    author="Ubiq Security, Inc.",
    author_email="support@ubiqsecurity.com",
    description="Python client library for accessing Ubiq Platform",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/ubiqsecurity/ubiq-python",
    packages=setuptools.find_packages(),
    install_requires=install_requires,
    license="Free To Use But Restricted",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: Free To Use But Restricted",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Security",
        "Topic :: Security :: Cryptography"
    ],
    python_requires='>=3.5',
)




#!/usr/bin/env python3
#
# Copyright 2020 Ubiq Security, Inc., Proprietary and All Rights Reserved.
#
# NOTICE:  All information contained herein is, and remains the property
# of Ubiq Security, Inc. The intellectual and technical concepts contained
# herein are proprietary to Ubiq Security, Inc. and its suppliers and may be
# covered by U.S. and Foreign Patents, patents in process, and are
# protected by trade secret or copyright law. Dissemination of this
# information or reproduction of this material is strictly forbidden
# unless prior written permission is obtained from Ubiq Security, Inc.
#
# Your use of the software is expressly conditioned upon the terms
# and conditions available at:
#
#     https://ubiqsecurity.com/legal
#


import cryptography.hazmat as crypto
from cryptography.hazmat.primitives.ciphers import (
    Cipher, algorithms, modes)

import os

class algorithm:
    """Internal representation of supported algorithm(s)

    This class is mostly a "front", allowing names and id's of algorithms
    to be seamlessly translated to code/function calls. A caller can
    create an algorithm object, supplying either the name of the algorithm
    or its numerical id, and the created object can be used to perform
    encryptions or decryptions using the algorithm specified.
    """

    #
    # names and numbers must be unique, individually. that is, using the
    # same number with two different names or vice versa is illegal. also,
    # do NOT alter the names or numbers of existing entries as that will
    # invalidate data that has already used the existing entry
    #
    # entries are in the format:
    # (algorithm-numeric-id, 'algorithm-text-name'):
    #     (algorithm-class, mode-class, key-length, iv-length, tag-length)
    # with all lengths specified in bytes
    #
    _ALGORITHM = {
        (0, 'aes-256-gcm'): (crypto.primitives.ciphers.algorithms.AES,
                             crypto.primitives.ciphers.modes.GCM,
                             32, 12, 16),
    }

    def __init__(self, ident):
        """
        ident:
            The name or numerical id of an algorithm
        """
        try:
            self.len = {}
            ((self.id, self.name),
             (self._algo, self._mode,
              self.len['key'], self.len['iv'], self.len['tag'])) = next(
                  ((k, v) for k, v in self._ALGORITHM.items() if ident in k))
        except:
            raise RuntimeError('algorithm (' + str(ident) + ') not found')

    def encryptor(self, key, iv=None):
        """
        key:
            A byte string containing the key to be used with this encryption
        iv:
            If the caller specifies the initialization vector, it must be
            the correct length and, if so, will be used. If it is not
            specified, the function will generate a new one

        returns:
            The encryptor object and the initialization vector
        """
        if len(key) != self.len['key']:
            raise RuntimeError('invalid key length')
        if iv is not None and len(iv) != self.len['iv']:
            raise RuntimeError('invalid initialization vector length')

        if iv is None:
            iv = os.urandom(self.len['iv'])

        return (
            crypto.primitives.ciphers.Cipher(
                self._algo(key),
                self._mode(iv),
                backend=crypto.backends.default_backend()).encryptor(),
            iv)

    def decryptor(self, key, iv):
        """
        key:
            A byte string containing the key to be used with this decryption
        iv:
            The initialization vector used when the data was encrypted

        Note that if a tag was used with the encryption, the caller must
        use the 'finalize_with_tag' function to verify the decryption.
        """
        if len(key) != self.len['key']:
            raise RuntimeError('invalid key length')
        if len(iv) != self.len['iv']:
            raise RuntimeError('invalid initialization vector length')

        if self.len['tag'] == 0:
            mode = self._mode(iv)
        else:
            mode = self._mode(iv, None, self.len['tag'])

        return crypto.primitives.ciphers.Cipher(
            self._algo(key),
            mode,
            backend=crypto.backends.default_backend()).decryptor()
